# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command lists the models in Model Garden and their supported functionalities."""

from googlecloudsdk.api_lib.ai.model_garden import client as client_mg
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as c_exceptions
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai.region_util import (
    _IsDefaultUniverse,
)
from googlecloudsdk.core import properties

_SHORT_NAME_FORMAT = (
    'format("{0:s}@{1:s}/{2:s}", name, versionId,'
    ' name.regex("publishers/hf-.*", "@hf", "@mg")).sub("publishers/hf-",'
    ' "").sub("publishers/", "").sub("models/", "").sub("@001/@hf", "").'
    ' sub("/@mg", ""):sort=1'
)
_FULL_RESOURCE_NAME_FORMAT = 'format("{0:s}@{1:s}", name, versionId):sort=1'
_MODEL_ID_LABEL = ':label=MODEL_ID'

_CAN_DEPLOY_FILTER = 'supportedActions.multiDeployVertex.yesno(yes=Yes)'
_CAN_DEPLOY_LABEL = ':label=CAN_DEPLOY'

_CAN_PREDICT_FILTER = 'publisherModelTemplate.yesno(yes=Yes)'
_CAN_PREDICT_LABEL = ':label=CAN_PREDICT'

_DEFAULT_TABLE_FORMAT = (
    f'table({_SHORT_NAME_FORMAT}{_MODEL_ID_LABEL},'
    f' {_CAN_DEPLOY_FILTER}{_CAN_DEPLOY_LABEL},'
    f' {_CAN_PREDICT_FILTER}{_CAN_PREDICT_LABEL})'
)
_SHORT_MODEL_NAME_ONLY_TABLE_FORMAT = (
    f'table({_SHORT_NAME_FORMAT}{_MODEL_ID_LABEL})'
)
_FULL_RESOURCE_NAME_ONLY_TABLE_FORMAT = (
    f'table({_FULL_RESOURCE_NAME_FORMAT}{_MODEL_ID_LABEL})'
)
_FULL_RESOURCE_NAME_TABLE_FORMAT = (
    f'table({_FULL_RESOURCE_NAME_FORMAT}{_MODEL_ID_LABEL},'
    f' {_CAN_DEPLOY_FILTER}{_CAN_DEPLOY_LABEL},'
    f' {_CAN_PREDICT_FILTER}{_CAN_PREDICT_LABEL})'
)


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.UniverseCompatible
class List(base.ListCommand):
  """List the publisher models in Model Garden.

  This command lists either all models in Model Garden or all Hugging
  Face models supported by Model Garden.

  ## EXAMPLES

  To list all models in Model Garden, run:

    $ gcloud ai model-garden models list

  To list Hugging Face models that can be deployed in Model Garden, run:

    $ gcloud ai model-garden models list --can-deploy-hugging-face-models

  To list models with `gemma` in their names, run:

    $ gcloud ai model-garden models list --model-filter=gemma

  Note: Since the number of Hugging Face models is large, the default limit is
  set to 500 with a page size of 100 when listing supported Hugging Face models.
  To override the limit or page size, specify the --limit or --page-size flags,
  respectively. To list all models in Model Garden, use `--limit=unlimited`.
  """

  @staticmethod
  def Args(parser):
    # The default display format is set in Run() rather than here so that an
    # explicit --format flag (e.g., --format="json(field)") is used as-is
    # without being composed with the default table projection. Format
    # composition causes default table columns to leak into json(...) output via
    # shared parent paths in the projection tree.
    parser.add_argument(
        '--can-deploy-hugging-face-models',
        action='store_true',
        default=False,
        required=False,
        help='Whether to only list Hugging Face models that can be deployed.',
    )
    parser.add_argument(
        '--model-filter',
        action='store',
        default=None,
        required=False,
        help=(
            'Filter to apply to the model names or the display names of the'
            ' list of models.'
        ),
    )
    parser.add_argument(
        '--full-resource-name',
        action='store_true',
        default=False,
        required=False,
        help='Whether to return the full resource name of the model.',
    )
    base.URI_FLAG.RemoveFromParser(parser)
    base.LIMIT_FLAG.SetDefault(parser, 1000)

  def Run(self, args):
    version = constants.BETA_VERSION

    # Set the default display format only when the user hasn't provided an
    # explicit --format.  Without this guard, the default table(...) projection
    # is composed with the explicit format, and the projection tree's INNER
    # flag on shared parent paths (e.g. supportedActions) causes sibling
    # branches from the table projection (e.g. multiDeployVertex) to leak
    # into json(...) output.
    if not args.IsSpecified('format'):
      if args.can_deploy_hugging_face_models:
        args.GetDisplayInfo().AddFormat(
            _FULL_RESOURCE_NAME_ONLY_TABLE_FORMAT
            if args.full_resource_name
            else _SHORT_MODEL_NAME_ONLY_TABLE_FORMAT
        )
      elif args.full_resource_name:
        args.GetDisplayInfo().AddFormat(_FULL_RESOURCE_NAME_TABLE_FORMAT)
      else:
        args.GetDisplayInfo().AddFormat(_DEFAULT_TABLE_FORMAT)

    # Set the default page size to 100 if the user requests to list supported
    # Hugging Face models, since there are tens of thousands of Hugging Face
    # models and the call will take a long time.
    if args.can_deploy_hugging_face_models:
      if args.page_size is None:
        args.page_size = 100

    if _IsDefaultUniverse():
      api_region = 'us-central1'
    else:
      api_region = properties.VALUES.ai.region.Get()
      if not api_region:
        raise c_exceptions.RequiredArgumentException(
            'ai/region',
            'The [ai/region] property must be set in this environment.',
        )
    with endpoint_util.AiplatformEndpointOverrides(
        version, region=api_region, use_info_log=True
    ):
      mg_client = client_mg.ModelGardenClient(version)
      return mg_client.ListPublisherModels(
          limit=args.limit,
          batch_size=args.page_size,
          list_hf_models=args.can_deploy_hugging_face_models,
          model_filter=args.model_filter,
      )



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to get credentials of an Attached cluster."""


from googlecloudsdk.api_lib.container.gkemulticloud import attached as api_util
from googlecloudsdk.api_lib.container.gkemulticloud import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.attached import resource_args
from googlecloudsdk.command_lib.container.gkemulticloud import endpoint_util
from googlecloudsdk.command_lib.container.gkemulticloud import flags
from googlecloudsdk.command_lib.container.gkemulticloud import kubeconfig
from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
@base.Deprecate(
    is_removed=False,
    warning=(
        'This command is deprecated. Use `gcloud container fleet memberships'
        ' get-credentials` instead.'
    ),
)
class GetCredentials(base.Command):
  """Get credentials of an Attached cluster."""

  detailed_help = {
      'EXAMPLES': kubeconfig.COMMAND_EXAMPLE,
      'DESCRIPTION': kubeconfig.COMMAND_DESCRIPTION.format(
          cluster_type='Attached cluster'
      ),
  }

  @staticmethod
  def Args(parser):
    resource_args.AddAttachedClusterResourceArg(parser, 'to get credentials')
    flags.AddAuthProviderCmdPath(parser)

  def Run(self, args):
    """Runs the get-credentials command."""
    with endpoint_util.GkemulticloudEndpointOverride(
        resource_args.ParseAttachedClusterResourceArg(args).locationsId,
        self.ReleaseTrack(),
    ):
      cluster_ref = resource_args.ParseAttachedClusterResourceArg(args)
      cluster_client = api_util.ClustersClient()
      log.status.Print('Fetching cluster endpoint and auth data.')
      resp = cluster_client.Get(cluster_ref)
      if (
          resp.state
          != util.GetMessagesModule().GoogleCloudGkemulticloudV1AttachedCluster.StateValueValuesEnum.RUNNING
      ):
        log.warning(
            kubeconfig.NOT_RUNNING_MSG.format(cluster_ref.attachedClustersId)
        )
      context = kubeconfig.GenerateContext(
          'attached',
          cluster_ref.projectsId,
          cluster_ref.locationsId,
          cluster_ref.attachedClustersId,
      )

      kubeconfig.GenerateAttachedKubeConfig(
          resp,
          cluster_ref.attachedClustersId,
          context,
          args.auth_provider_cmd_path,
      )



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Implementation of delete command for Feature Configs."""

from googlecloudsdk.api_lib.storage import feature_config_api
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class Delete(base.Command):
  """Delete a Feature Config."""

  detailed_help = {
      'DESCRIPTION': (
          """
       Delete an existing Feature Config.
      """
      ),
      'EXAMPLES': (
          """
      To delete a Feature Config named "my_config":

         $ {command} my_config
      """
      ),
  }

  @classmethod
  def Args(cls, parser):
    parser.add_argument(
        'CONFIG_ID',
        help='The ID of the feature configuration to delete.',
    )
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    client = feature_config_api.FeatureConfigApi()
    project = properties.VALUES.core.project.Get(required=True)
    name = (
        f'projects/{project}/locations/global/featureConfigs/{args.CONFIG_ID}'
    )
    return client.delete_feature_config(name)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Contains logic for finalizing composite uploads."""


import os

from googlecloudsdk.command_lib.storage import errors
from googlecloudsdk.command_lib.storage import manifest_util
from googlecloudsdk.command_lib.storage import tracker_file_util
from googlecloudsdk.command_lib.storage.tasks import compose_objects_task
from googlecloudsdk.command_lib.storage.tasks import task
from googlecloudsdk.command_lib.storage.tasks import task_util
from googlecloudsdk.command_lib.storage.tasks.cp import copy_util
from googlecloudsdk.command_lib.storage.tasks.cp import delete_temporary_components_task
from googlecloudsdk.core import properties


class FinalizeCompositeUploadTask(copy_util.ObjectCopyTaskWithExitHandler):
  """Composes and deletes object resources received as messages."""

  def __init__(
      self,
      expected_component_count,
      source_resource,
      destination_resource,
      delete_source=False,
      posix_to_set=None,
      print_created_message=False,
      random_prefix='',
      temporary_paths_to_clean_up=None,
      user_request_args=None,
  ):
    """Initializes task.

    Args:
      expected_component_count (int): Number of temporary components expected.
      source_resource (resource_reference.FileObjectResource): The local
        uploaded file.
      destination_resource (resource_reference.UnknownResource): Metadata for
        the final composite object.
      delete_source (bool): If copy completes successfully, delete the source
        object afterwards.
      posix_to_set (PosixAttributes|None): See parent class.
      print_created_message (bool): See parent class.
      random_prefix (str): Random id added to component names.
      temporary_paths_to_clean_up (str): Paths to remove after the composite
        upload completes. This may include a temporary gzipped version of the
        source, or symlink placeholders.
      user_request_args (UserRequestArgs|None): See parent class.
    """
    super(FinalizeCompositeUploadTask, self).__init__(
        source_resource,
        destination_resource,
        posix_to_set=posix_to_set,
        print_created_message=print_created_message,
        user_request_args=user_request_args,
    )
    self._expected_component_count = expected_component_count
    self._delete_source = delete_source
    self._random_prefix = random_prefix
    self._temporary_paths_to_clean_up = temporary_paths_to_clean_up

  def execute(self, task_status_queue=None):
    uploaded_components = [
        message.payload
        for message in self.received_messages
        if message.topic == task.Topic.UPLOADED_COMPONENT
    ]

    if len(uploaded_components) != self._expected_component_count:
      raise errors.Error(
          'Temporary components were not uploaded correctly.'
          ' Please retry this upload.'
      )

    uploaded_objects = [
        component.object_resource
        for component in sorted(
            uploaded_components,
            key=lambda component: component.component_number)
    ]

    delete_source_objects = (
        properties.VALUES.storage.delete_source_objects_in_compose.GetBool()
    )
    compose_task = compose_objects_task.ComposeObjectsTask(
        uploaded_objects,
        self._destination_resource,
        original_source_resource=self._source_resource,
        posix_to_set=self._posix_to_set,
        user_request_args=self._user_request_args,
        delete_source_objects=delete_source_objects,
    )
    compose_task_output = compose_task.execute(
        task_status_queue=task_status_queue
    )

    result_resource = task_util.get_first_matching_message_payload(
        compose_task_output.messages, task.Topic.CREATED_RESOURCE
    )
    if result_resource:
      self._print_created_message_if_requested(result_resource)
      if self._send_manifest_messages:
        manifest_util.send_success_message(
            task_status_queue,
            self._source_resource,
            self._destination_resource,
            md5_hash=result_resource.md5_hash,
        )

    # After a successful compose call, we consider the upload complete and can
    # delete tracker files.
    tracker_file_path = tracker_file_util.get_tracker_file_path(
        self._destination_resource.storage_url,
        tracker_file_util.TrackerFileType.PARALLEL_UPLOAD,
        source_url=self._source_resource,
    )
    tracker_file_util.delete_tracker_file(tracker_file_path)

    for path in self._temporary_paths_to_clean_up or []:
      os.remove(path)
    if self._delete_source:
      # Delete original source file.
      os.remove(self._source_resource.storage_url.resource_name)

    return task.Output(
        additional_task_iterators=[
            [
                delete_temporary_components_task.DeleteTemporaryComponentsTask(
                    self._source_resource,
                    self._destination_resource,
                    self._random_prefix,
                )
            ]
        ],
        messages=None,
    )

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (
        self._expected_component_count == other._expected_component_count
        and self._source_resource == other._source_resource
        and self._destination_resource == other._destination_resource
        and self._random_prefix == other._random_prefix
        and self._temporary_paths_to_clean_up
        == other._temporary_paths_to_clean_up
        and self._user_request_args == other._user_request_args
    )



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""AddIamPolicyBinding command for the Resource Manager - Tag Keys CLI."""


import http.client

from googlecloudsdk.api_lib.resource_manager import tags
from googlecloudsdk.api_lib.util import http_retry
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.resource_manager import tag_arguments as arguments
from googlecloudsdk.command_lib.resource_manager import tag_utils


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class AddIamPolicyBinding(base.Command):
  """Adds a policy binding to the IAM policy of a TagKey.

     Adds the IAM policy binding for a TagKey resource given the binding
     and an identifier for the TagKey. The identifier can be the TagKey's
     parent/short name or the TagKey's ID in the form:
     tagKeys/{numeric_id}.
  """

  detailed_help = {
      'EXAMPLES':
          """
          To add an IAM policy binding for the role of 'roles/editor' for the
          user 'test-user@gmail.com' on the Tag Key 'tagKeys/123', run:

            $ {command} tagKeys/123 --member='user:test-user@gmail.com' --role='roles/editor'

          To add an IAM policy binding for a Tag Key with the name 'env' under
          'organization/456' for the role of 'roles/resourcemanager.tagUser' for
          the user 'test-user@gmail.com', run:

            $ {command} 456/env --member='user:test-user@gmail.com' --role='roles/resourcemanager.tagUser'

          See https://cloud.google.com/iam/docs/managing-policies for details of
          policy role and member types.

          """
  }

  @staticmethod
  def Args(parser):
    arguments.AddResourceNameArgToParser(parser)
    iam_util.AddArgsForAddIamPolicyBinding(parser, add_condition=True)

  # Allow for retries due to etag-based optimistic concurrency control
  @http_retry.RetryOnHttpStatus(http.client.CONFLICT)
  def Run(self, args):
    service = tags.TagKeysService()
    messages = tags.TagMessages()

    if args.RESOURCE_NAME.find('tagKeys/') == 0:
      tag_key = args.RESOURCE_NAME
    else:
      tag_key = tag_utils.GetNamespacedResource(
          args.RESOURCE_NAME, tag_utils.TAG_KEYS
      ).name

    get_iam_policy_req = (
        messages.CloudresourcemanagerTagKeysGetIamPolicyRequest(
            resource=tag_key))
    policy = service.GetIamPolicy(get_iam_policy_req)
    condition = iam_util.ValidateAndExtractConditionMutexRole(args)
    iam_util.AddBindingToIamPolicyWithCondition(messages.Binding, messages.Expr,
                                                policy, args.member, args.role,
                                                condition)

    set_iam_policy_request = messages.SetIamPolicyRequest(policy=policy)
    request = messages.CloudresourcemanagerTagKeysSetIamPolicyRequest(
        resource=tag_key, setIamPolicyRequest=set_iam_policy_request)
    result = service.SetIamPolicy(request)
    iam_util.LogSetIamPolicy(tag_key, 'TagKey')
    return result



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List incompatible features for GAE to CR migration."""
import logging
from os import path as os_path
from typing import Dict, List, Mapping, Sequence
from apitools.base.protorpclite import messages
from apitools.base.py import encoding
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.common import util
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.config import feature_helper
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import properties
from googlecloudsdk.core import yaml


_TEMPLATE_PATH = os_path.join(os_path.dirname(__file__), '../config/')


class IncompatibleFeaturesFoundError(exceptions.Error):
  """An error that is raised when incompatible features are found."""


def check_for_urlmap_conditions(
    url_maps: List[any], input_type: feature_helper.InputType
) -> bool:
  """Checks if any UrlMap in the list has urlRegex='.*' and scriptPath='auto'.

  Args:
      url_maps: A list of UrlMap objects.
      input_type: The input type of the app.yaml file.
  Returns:
      True if all UrlMap matches the conditions, False otherwise.
  """
  for url_map in url_maps:
    if (
        input_type == feature_helper.InputType.ADMIN_API
        and (
            url_map.get('urlRegex') == '.*' or url_map.get('urlRegex') == '/.*'
        )
        and url_map.get('script', {}).get('scriptPath')
        in ['auto', 'PLACEHOLDER']
    ):
      continue
    elif (
        input_type == feature_helper.InputType.APP_YAML
        and (url_map.get('url') == '.*' or url_map.get('url') == '/.*')
        and url_map.get('script') in ['auto', 'PLACEHOLDER']
    ):
      continue
    else:
      return False
  return True


def get_length(val: any) -> int:
  """Returns the length of the given value."""
  if isinstance(val, list):
    return len(val)
  elif isinstance(val, str):
    return len(val)
  elif isinstance(val, bytes):
    return len(val)
  else:
    return 0


def list_incompatible_features(
    input_data: Mapping[str, any],
    input_type: feature_helper.InputType,
    appyaml: str,
    service: str,
    version: str,
) -> None:
  """Lists incompatible features for GAE to Cloud Run migration.

  This function checks for incompatible features in either a local app.yaml file
  or a deployed App Engine version.

  Args:
      input_data: The input data, containing Message objects.
      input_type: The input type of the input data.
      appyaml: The path to the app.yaml file.
      service: The service name.
      version: The version name.
  """
  if not input_type or not input_data:
    return
  incompatible_list = _check_for_incompatibility(input_data, input_type)
  input_name = _generate_input_name(input_type, appyaml, service, version)
  _generate_output(incompatible_list, input_type, input_name)


def _generate_input_name(
    input_type: feature_helper.InputType,
    appyaml: str,
    service: str,
    version: str,
) -> str:
  """Generates the input name for the input type."""
  if input_type == feature_helper.InputType.APP_YAML:
    return appyaml
  project_id = properties.VALUES.core.project.Get()
  return f'{project_id}/{service}/{version}'


def _convert_message_to_dict(data: Mapping[str, any]):
  """Recursively converts apitools messages.Message objects to dictionaries.

  Args:
      data: The input data, potentially containing Message objects.

  Returns:
      A new structure with Message objects converted to dictionaries.
  """
  if isinstance(data, messages.Message):
    dict_data = encoding.MessageToDict(data)
    return _convert_message_to_dict(dict_data)
  elif isinstance(data, dict):
    new_dict = {}
    for k, v in data.items():
      new_dict[k] = _convert_message_to_dict(v)
    return new_dict
  elif isinstance(data, list):
    return [_convert_message_to_dict(elem) for elem in data]
  else:
    return data


def _check_for_incompatibility(
    input_data: Mapping[str, any], input_type: feature_helper.InputType
) -> Sequence[any]:
  """Check for incompatibility features in the input yaml."""
  incompatible_list: List[any] = []
  feature_config = feature_helper.get_feature_config()
  unsupported_features = feature_helper.get_feature_list_by_input_type(
      input_type, feature_config.unsupported
  )
  range_limited_features = feature_helper.get_feature_list_by_input_type(
      input_type, feature_config.range_limited
  )
  value_restricted_features = feature_helper.get_feature_list_by_input_type(
      input_type, feature_config.value_limited
  )
  processed_input_data = input_data
  if input_type == feature_helper.InputType.ADMIN_API:
    processed_input_data = _convert_message_to_dict(input_data)
  input_key_value_pairs = util.flatten_keys(processed_input_data, '')
  for key, val in input_key_value_pairs.items():
    # Check for unsupported features.
    if key in unsupported_features:
      if (
          key.startswith('inboundServices')
          or key.startswith('inbound_services')
      ) and get_length(val) > 0:
        incompatible_list.append(unsupported_features[key])
        continue
      if (
          key.startswith('errorHandlers') or key.startswith('error_handlers')
      ) and get_length(val) > 0:
        incompatible_list.append(unsupported_features[key])
        continue
      if key == 'handlers' and not check_for_urlmap_conditions(val, input_type):
        incompatible_list.append(unsupported_features[key])
        continue
      if key in ['app_engine_apis', 'appEngineApis'] and val:
        incompatible_list.append(unsupported_features[key])
        continue
      if (
          key in ['resources.disk_size_gb', 'resources.diskGb']
          and val is not None
      ):
        try:
          if float(val) > 0:
            incompatible_list.append(unsupported_features[key])
        except (ValueError, TypeError):
          pass
        continue
    # Check for basic scaling features.
    if key in [
        'basic_scaling.max_instances',
        'basicScaling.maxInstances',
        'basic_scaling.min_instances',
        'basicScaling.minInstances',
    ]:
      incompatible_list.append(range_limited_features[key])
      continue
    # Check for range_limited features.
    if key in range_limited_features:
      if not range_limited_features[key].validate(val):
        incompatible_list.append(range_limited_features[key])
    # Check for value_restricted features.
    if key in value_restricted_features:
      # TODO: b/493877807 - Better handle runtime validation for flex.
      if util.is_flex_env(input_data) and key.startswith('runtime'):
        continue
      if not value_restricted_features[key].validate(key, val):
        incompatible_list.append(value_restricted_features[key])
  return incompatible_list


def _generate_output(
    incompatible_features: List[feature_helper.UnsupportedFeature],
    input_type: feature_helper.InputType,
    input_name: str,
) -> None:
  """Generate readable output for features compatibility check result."""
  print(f'List incompatible features output for {input_name}:\n')
  logging.info('list-incompatible-features output for %s:\n', input_name)
  if not incompatible_features:
    print('No incompatibilities found.\n')
    logging.info('No incompatibilities found.\n')
    return
  major_features = []
  minor_features = []
  for feature in incompatible_features:
    if feature.severity == 'major':
      major_features.append(feature)
    elif feature.severity == 'minor':
      minor_features.append(feature)
  if minor_features:
    print(
        f'Summary:\nminor: {len(minor_features)}\n'
        f'incompatible_features\n{yaml.dump(_get_display_features(minor_features, input_type))}\n'
    )
    logging.info(
        'Summary:\nminor: %s\nincompatible_features\n%s',
        len(minor_features),
        yaml.dump(_get_display_features(minor_features, input_type)),
    )
  if major_features:
    display_major_features = yaml.dump(
        _get_display_features(major_features, input_type)
    )
    error_message = (
        f'Summary:\nmajor: {len(major_features)}\n'
        f'incompatible_features\n{display_major_features}\n '
    )
    raise IncompatibleFeaturesFoundError(
        error_message
    )


def _get_display_features(
    features: List[feature_helper.UnsupportedFeature],
    input_type: feature_helper.InputType
) -> Sequence[Dict[str, str]]:
  """Converts UnsupportedFeature objects to a display format.

  Args:
      features: A list of UnsupportedFeature objects.
      input_type: The input type of the app.yaml file.

  Returns:
      A list of dictionaries suitable for printing.
  """
  features_display = []
  for feature in features:
    features_display.append({
        'message': feature.reason,
        'category': feature.path[input_type.value],
        'severity': feature.severity,
    })
  return features_display



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Backup and DR Service Config client."""


from googlecloudsdk.api_lib.backupdr import util


class ServiceConfigClient(util.BackupDrClientBase):
  """Cloud Backup and DR Service Config client."""

  def __init__(self, api_version=util.DEFAULT_API_VERSION):
    super(ServiceConfigClient, self).__init__(api_version=api_version)
    self.service = self.client.projects_locations_serviceConfig

  def Init(self, location, resource_type, cloudsql_edition):
    """Calls the Backup and DR Initialize service.

    Args:
      location: location of the service config.
      resource_type: resource type for which the service config is being
        initialized.
      cloudsql_edition: (Required only for Cloud SQL instances, ignored
        otherwise) The edition of the Cloud SQL instance. Possible values:
        "ENTERPRISE" - Enterprise edition. "ENTERPRISE_PLUS" - Enterprise Plus
        edition.

    Returns:
      A long running operation
    """
    name = f'{location}/serviceConfig'
    if cloudsql_edition:
      cloudsql_edition_enum = self.messages.CloudSqlInstanceInitializationConfig.EditionValueValuesEnum(
          cloudsql_edition
      )
      request = (
          self.messages.BackupdrProjectsLocationsServiceConfigInitializeRequest(
              name=name,
              initializeServiceRequest=self.messages.InitializeServiceRequest(
                  resourceType=resource_type,
                  cloudSqlInstanceInitializationConfig=(
                      self.messages.CloudSqlInstanceInitializationConfig(
                          edition=cloudsql_edition_enum,
                      )
                  ),
              ),
          )
      )
    else:
      request = (
          self.messages.BackupdrProjectsLocationsServiceConfigInitializeRequest(
              name=name,
              initializeServiceRequest=self.messages.InitializeServiceRequest(
                  resourceType=resource_type,
              ),
          )
      )
    return self.service.Initialize(request)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update command for the resource manager - Tag Keys CLI."""


from googlecloudsdk.api_lib.resource_manager import tags
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.resource_manager import operations
from googlecloudsdk.command_lib.resource_manager import tag_arguments as arguments
from googlecloudsdk.command_lib.resource_manager import tag_utils


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Update(base.Command):
  """Updates the specified TagKey resource's description.

    Updates the TagKey's description given the TagKey's parent and short name
    or the TagKey's numeric id.
  """

  detailed_help = {
      'EXAMPLES':
          """
          To update a TagKey with id ``123'', run:

            $ {command} tagKeys/123 --description=foobar --allowed-values-regex=.*

          To update a TagKey named ``env'' under organization ``456'',
          run:

            $ {command} 456/env --description=foobar --allowed-values-regex=.*
          """
  }

  @classmethod
  def Args(cls, parser):
    arguments.AddResourceNameArgToParser(parser)
    arguments.AddAsyncArgToParser(parser)
    arguments.AddDescriptionArgToParser(parser)
    arguments.AddAllowedValuesRegexArgToParser(parser)
    if cls.ReleaseTrack() == base.ReleaseTrack.ALPHA:
      arguments.AddSemanticsMutexGroupArgs(parser)

  def Run(self, args):
    service = tags.TagKeysService()
    messages = tags.TagMessages()

    if args.RESOURCE_NAME.find('tagKeys/') == 0:
      tag_key = tag_utils.GetResource(
          args.RESOURCE_NAME, tag_utils.TAG_KEYS)
    else:
      tag_key = tag_utils.GetNamespacedResource(
          args.RESOURCE_NAME, tag_utils.TAG_KEYS
      )

    update_masks = []
    if 'allowed_values_regex' in args and args.IsSpecified(
        'allowed_values_regex'
    ):
      tag_key.allowedValuesRegex = args.allowed_values_regex
      update_masks.append('allowed_values_regex')

    if args.IsSpecified('description'):
      tag_key.description = args.description
      update_masks.append('description')

    if 'add_semantics' in args and args.IsSpecified('add_semantics'):
      tag_key.semantics = args.add_semantics
      update_masks.append('semantics')
    elif 'remove_semantics' in args and args.IsSpecified('remove_semantics'):
      update_masks.append('semantics')
      tag_key.semantics = None

    update_request = messages.CloudresourcemanagerTagKeysPatchRequest(
        name=tag_key.name, tagKey=tag_key, updateMask=','.join(update_masks))
    op = service.Patch(update_request)

    if args.async_:
      return op

    return operations.WaitForOperation(
        op,
        'Waiting for TagKey [{}] to be updated'.format(tag_key.name),
        service=service)



"""Generated client library for designcenter version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.designcenter.v1 import designcenter_v1_messages as messages


class DesigncenterV1(base_api.BaseApiClient):
  """Generated client library for service designcenter version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://designcenter.googleapis.com/'
  MTLS_BASE_URL = 'https://designcenter.mtls.googleapis.com/'

  _PACKAGE = 'designcenter'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'DesigncenterV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new designcenter handle."""
    url = url or self.BASE_URL
    super(DesigncenterV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_spaces_applicationTemplates_components_connections = self.ProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsService(self)
    self.projects_locations_spaces_applicationTemplates_components = self.ProjectsLocationsSpacesApplicationTemplatesComponentsService(self)
    self.projects_locations_spaces_applicationTemplates_policies = self.ProjectsLocationsSpacesApplicationTemplatesPoliciesService(self)
    self.projects_locations_spaces_applicationTemplates_revisions = self.ProjectsLocationsSpacesApplicationTemplatesRevisionsService(self)
    self.projects_locations_spaces_applicationTemplates_systemRevisions = self.ProjectsLocationsSpacesApplicationTemplatesSystemRevisionsService(self)
    self.projects_locations_spaces_applicationTemplates = self.ProjectsLocationsSpacesApplicationTemplatesService(self)
    self.projects_locations_spaces_applications = self.ProjectsLocationsSpacesApplicationsService(self)
    self.projects_locations_spaces_catalogs_shares = self.ProjectsLocationsSpacesCatalogsSharesService(self)
    self.projects_locations_spaces_catalogs_templates_revisions = self.ProjectsLocationsSpacesCatalogsTemplatesRevisionsService(self)
    self.projects_locations_spaces_catalogs_templates = self.ProjectsLocationsSpacesCatalogsTemplatesService(self)
    self.projects_locations_spaces_catalogs = self.ProjectsLocationsSpacesCatalogsService(self)
    self.projects_locations_spaces_sharedTemplates_revisions = self.ProjectsLocationsSpacesSharedTemplatesRevisionsService(self)
    self.projects_locations_spaces_sharedTemplates = self.ProjectsLocationsSpacesSharedTemplatesService(self)
    self.projects_locations_spaces = self.ProjectsLocationsSpacesService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.

      Args:
        request: (DesigncenterProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='designcenter.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='DesigncenterProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (DesigncenterProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (DesigncenterProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (DesigncenterProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='designcenter.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v1/{+name}/operations',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_applicationTemplates_components_connections resource."""

    _NAME = 'projects_locations_spaces_applicationTemplates_components_connections'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a connection.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Connection) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components/{componentsId}/connections',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.connections.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['connectionId'],
        relative_path='v1/{+parent}/connections',
        request_field='connection',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsCreateRequest',
        response_type_name='Connection',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a connection.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components/{componentsId}/connections/{connectionsId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.connections.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details about a connection.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Connection) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components/{componentsId}/connections/{connectionsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.connections.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsGetRequest',
        response_type_name='Connection',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists connections in an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListConnectionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components/{componentsId}/connections',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.connections.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/connections',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsListRequest',
        response_type_name='ListConnectionsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a connection.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Connection) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components/{componentsId}/connections/{connectionsId}',
        http_method='PATCH',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.connections.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='connection',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsConnectionsPatchRequest',
        response_type_name='Connection',
        supports_download=False,
    )

  class ProjectsLocationsSpacesApplicationTemplatesComponentsService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_applicationTemplates_components resource."""

    _NAME = 'projects_locations_spaces_applicationTemplates_components'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesApplicationTemplatesComponentsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a component.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Component) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['componentId'],
        relative_path='v1/{+parent}/components',
        request_field='component',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsCreateRequest',
        response_type_name='Component',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a component.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components/{componentsId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details about a Component.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Component) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components/{componentsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsGetRequest',
        response_type_name='Component',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists components in an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListComponentsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/components',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsListRequest',
        response_type_name='ListComponentsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a component.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Component) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/components/{componentsId}',
        http_method='PATCH',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.components.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='component',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesComponentsPatchRequest',
        response_type_name='Component',
        supports_download=False,
    )

  class ProjectsLocationsSpacesApplicationTemplatesPoliciesService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_applicationTemplates_policies resource."""

    _NAME = 'projects_locations_spaces_applicationTemplates_policies'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesApplicationTemplatesPoliciesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a policy for an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DesignCenterPolicy) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/policies',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.policies.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['policyId'],
        relative_path='v1/{+parent}/policies',
        request_field='designCenterPolicy',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesCreateRequest',
        response_type_name='DesignCenterPolicy',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a policy.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/policies/{policiesId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.policies.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Policy related APIs. Get a policy for an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DesignCenterPolicy) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/policies/{policiesId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.policies.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesGetRequest',
        response_type_name='DesignCenterPolicy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists policies for an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListPoliciesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/policies',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.policies.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/policies',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesListRequest',
        response_type_name='ListPoliciesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a policy.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DesignCenterPolicy) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/policies/{policiesId}',
        http_method='PATCH',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.policies.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='designCenterPolicy',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesPoliciesPatchRequest',
        response_type_name='DesignCenterPolicy',
        supports_download=False,
    )

  class ProjectsLocationsSpacesApplicationTemplatesRevisionsService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_applicationTemplates_revisions resource."""

    _NAME = 'projects_locations_spaces_applicationTemplates_revisions'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesApplicationTemplatesRevisionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Delete(self, request, global_params=None):
      r"""Deletes an application template revision.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/revisions/{revisionsId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.revisions.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Export(self, request, global_params=None):
      r"""Exports Terraform files for an application template revision. The generated modules can have remote sources or local sources downloaded from remote. This is an ASYNCHRONOUS operation that returns a Long Running Operation.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsExportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Export')
      return self._RunMethod(
          config, request, global_params=global_params)

    Export.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/revisions/{revisionsId}:export',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.revisions.export',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:export',
        request_field='exportApplicationTemplateRevisionIaCRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsExportRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Generate(self, request, global_params=None):
      r"""Generates Terraform files for an application template revision. This is a SYNCHRONOUS operation and returns the generated IaC directly in the response. It does NOT download external module sources; it only uses remote sources.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsGenerateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GenerateApplicationTemplateRevisionIaCResponse) The response message.
      """
      config = self.GetMethodConfig('Generate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Generate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/revisions/{revisionsId}:generate',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.revisions.generate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:generate',
        request_field='generateApplicationTemplateRevisionIaCRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsGenerateRequest',
        response_type_name='GenerateApplicationTemplateRevisionIaCResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details about an application template revision.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ApplicationTemplateRevision) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/revisions/{revisionsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.revisions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsGetRequest',
        response_type_name='ApplicationTemplateRevision',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists application template revisions in a given application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListApplicationTemplateRevisionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/revisions',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.revisions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/revisions',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesRevisionsListRequest',
        response_type_name='ListApplicationTemplateRevisionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsSpacesApplicationTemplatesSystemRevisionsService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_applicationTemplates_systemRevisions resource."""

    _NAME = 'projects_locations_spaces_applicationTemplates_systemRevisions'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesApplicationTemplatesSystemRevisionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets the given application template system revision.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesSystemRevisionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ApplicationTemplateSystemRevision) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/systemRevisions/{systemRevisionsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.systemRevisions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesSystemRevisionsGetRequest',
        response_type_name='ApplicationTemplateSystemRevision',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists application template system revisions in a given application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesSystemRevisionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListApplicationTemplateSystemRevisionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}/systemRevisions',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.systemRevisions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/systemRevisions',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesSystemRevisionsListRequest',
        response_type_name='ListApplicationTemplateSystemRevisionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsSpacesApplicationTemplatesService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_applicationTemplates resource."""

    _NAME = 'projects_locations_spaces_applicationTemplates'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesApplicationTemplatesService, self).__init__(client)
      self._upload_configs = {
          }

    def Commit(self, request, global_params=None):
      r"""Commits an application template to create a new revision.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesCommitRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ApplicationTemplateRevision) The response message.
      """
      config = self.GetMethodConfig('Commit')
      return self._RunMethod(
          config, request, global_params=global_params)

    Commit.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}:commit',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.commit',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:commit',
        request_field='commitApplicationTemplateRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesCommitRequest',
        response_type_name='ApplicationTemplateRevision',
        supports_download=False,
    )

    def Create(self, request, global_params=None):
      r"""Creates an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ApplicationTemplate) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['applicationTemplateId'],
        relative_path='v1/{+parent}/applicationTemplates',
        request_field='applicationTemplate',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesCreateRequest',
        response_type_name='ApplicationTemplate',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Export(self, request, global_params=None):
      r"""Exports Terraform files for an application template. The generated modules can have remote sources or local sources downloaded from remote. This is an ASYNCHRONOUS operation that returns a Long Running Operation.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesExportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Export')
      return self._RunMethod(
          config, request, global_params=global_params)

    Export.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}:export',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.export',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:export',
        request_field='exportApplicationTemplateIaCRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesExportRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Generate(self, request, global_params=None):
      r"""Generates Terraform files for an application template. This is a SYNCHRONOUS operation and returns the generated IaC directly in the response. It does NOT download external module sources; it only uses remote sources.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesGenerateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GenerateApplicationTemplateIaCResponse) The response message.
      """
      config = self.GetMethodConfig('Generate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Generate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}:generate',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.generate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:generate',
        request_field='generateApplicationTemplateIaCRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesGenerateRequest',
        response_type_name='GenerateApplicationTemplateIaCResponse',
        supports_download=False,
    )

    def GenerateAssessmentReport(self, request, global_params=None):
      r"""Generate Assessment Reports of an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesGenerateAssessmentReportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('GenerateAssessmentReport')
      return self._RunMethod(
          config, request, global_params=global_params)

    GenerateAssessmentReport.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}:generateAssessmentReport',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.generateAssessmentReport',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:generateAssessmentReport',
        request_field='generateApplicationTemplateAssessmentReportRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesGenerateAssessmentReportRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details about an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ApplicationTemplate) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesGetRequest',
        response_type_name='ApplicationTemplate',
        supports_download=False,
    )

    def Import(self, request, global_params=None):
      r"""Imports to an existing application template from a given shared template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesImportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ApplicationTemplate) The response message.
      """
      config = self.GetMethodConfig('Import')
      return self._RunMethod(
          config, request, global_params=global_params)

    Import.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}:import',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.import',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:import',
        request_field='importApplicationTemplateRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesImportRequest',
        response_type_name='ApplicationTemplate',
        supports_download=False,
    )

    def ImportIaC(self, request, global_params=None):
      r"""Imports IaC for an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesImportIaCRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ImportApplicationTemplateIaCResponse) The response message.
      """
      config = self.GetMethodConfig('ImportIaC')
      return self._RunMethod(
          config, request, global_params=global_params)

    ImportIaC.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}:importIaC',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.importIaC',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:importIaC',
        request_field='importApplicationTemplateIaCRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesImportIaCRequest',
        response_type_name='ImportApplicationTemplateIaCResponse',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists application templates in a given space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListApplicationTemplatesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/applicationTemplates',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesListRequest',
        response_type_name='ListApplicationTemplatesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an application template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationTemplatesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ApplicationTemplate) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applicationTemplates/{applicationTemplatesId}',
        http_method='PATCH',
        method_id='designcenter.projects.locations.spaces.applicationTemplates.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='applicationTemplate',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationTemplatesPatchRequest',
        response_type_name='ApplicationTemplate',
        supports_download=False,
    )

  class ProjectsLocationsSpacesApplicationsService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_applications resource."""

    _NAME = 'projects_locations_spaces_applications'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesApplicationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates an application.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Application) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applications.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['applicationId'],
        relative_path='v1/{+parent}/applications',
        request_field='application',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsCreateRequest',
        response_type_name='Application',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes an application.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications/{applicationsId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.applications.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Deploy(self, request, global_params=None):
      r"""Deploys an application.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsDeployRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Deploy')
      return self._RunMethod(
          config, request, global_params=global_params)

    Deploy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications/{applicationsId}:deploy',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applications.deploy',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:deploy',
        request_field='deployApplicationRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsDeployRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def FetchAssessmentReport(self, request, global_params=None):
      r"""Fetches the assessment report of a deployed application if present.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsFetchAssessmentReportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FetchApplicationAssessmentReportResponse) The response message.
      """
      config = self.GetMethodConfig('FetchAssessmentReport')
      return self._RunMethod(
          config, request, global_params=global_params)

    FetchAssessmentReport.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications/{applicationsId}:fetchAssessmentReport',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applications.fetchAssessmentReport',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:fetchAssessmentReport',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsFetchAssessmentReportRequest',
        response_type_name='FetchApplicationAssessmentReportResponse',
        supports_download=False,
    )

    def Generate(self, request, global_params=None):
      r"""Generates Terraform files for an application.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsGenerateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GenerateApplicationIaCResponse) The response message.
      """
      config = self.GetMethodConfig('Generate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Generate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications/{applicationsId}:generate',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applications.generate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:generate',
        request_field='generateApplicationIaCRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsGenerateRequest',
        response_type_name='GenerateApplicationIaCResponse',
        supports_download=False,
    )

    def GenerateAssessmentReport(self, request, global_params=None):
      r"""Generate Assessment Reports of an application.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsGenerateAssessmentReportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('GenerateAssessmentReport')
      return self._RunMethod(
          config, request, global_params=global_params)

    GenerateAssessmentReport.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications/{applicationsId}:generateAssessmentReport',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applications.generateAssessmentReport',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:generateAssessmentReport',
        request_field='generateApplicationAssessmentReportRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsGenerateAssessmentReportRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details about an application.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Application) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications/{applicationsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applications.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsGetRequest',
        response_type_name='Application',
        supports_download=False,
    )

    def ImportIaC(self, request, global_params=None):
      r"""Imports IaC for an application resource.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsImportIaCRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ImportApplicationIaCResponse) The response message.
      """
      config = self.GetMethodConfig('ImportIaC')
      return self._RunMethod(
          config, request, global_params=global_params)

    ImportIaC.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications/{applicationsId}:importIaC',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applications.importIaC',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:importIaC',
        request_field='importApplicationIaCRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsImportIaCRequest',
        response_type_name='ImportApplicationIaCResponse',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists applications in a given space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListApplicationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.applications.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/applications',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsListRequest',
        response_type_name='ListApplicationsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an application.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Application) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications/{applicationsId}',
        http_method='PATCH',
        method_id='designcenter.projects.locations.spaces.applications.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='application',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsPatchRequest',
        response_type_name='Application',
        supports_download=False,
    )

    def Preview(self, request, global_params=None):
      r"""Previews deployment of an application.

      Args:
        request: (DesigncenterProjectsLocationsSpacesApplicationsPreviewRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Preview')
      return self._RunMethod(
          config, request, global_params=global_params)

    Preview.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/applications/{applicationsId}:preview',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.applications.preview',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:preview',
        request_field='previewApplicationRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesApplicationsPreviewRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsSpacesCatalogsSharesService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_catalogs_shares resource."""

    _NAME = 'projects_locations_spaces_catalogs_shares'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesCatalogsSharesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Shares a catalog from one space to another space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsSharesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/shares',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.catalogs.shares.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['shareId'],
        relative_path='v1/{+parent}/shares',
        request_field='share',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsSharesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a catalog share.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsSharesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/shares/{sharesId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.catalogs.shares.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsSharesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get catalog share object for the space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsSharesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Share) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/shares/{sharesId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.catalogs.shares.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsSharesGetRequest',
        response_type_name='Share',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists catalog share objects for the space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsSharesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSharesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/shares',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.catalogs.shares.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/shares',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsSharesListRequest',
        response_type_name='ListSharesResponse',
        supports_download=False,
    )

    def Sync(self, request, global_params=None):
      r"""Synchronizes a share and updates the destination workspace to reflect. reflect the most recent changes to the catalog.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsSharesSyncRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Sync')
      return self._RunMethod(
          config, request, global_params=global_params)

    Sync.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/shares/{sharesId}:sync',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.catalogs.shares.sync',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:sync',
        request_field='syncShareRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsSharesSyncRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsSpacesCatalogsTemplatesRevisionsService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_catalogs_templates_revisions resource."""

    _NAME = 'projects_locations_spaces_catalogs_templates_revisions'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesCatalogsTemplatesRevisionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a template revision.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/templates/{templatesId}/revisions',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.catalogs.templates.revisions.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['catalogTemplateRevisionId'],
        relative_path='v1/{+parent}/revisions',
        request_field='catalogTemplateRevision',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a template revision.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/templates/{templatesId}/revisions/{revisionsId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.catalogs.templates.revisions.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details about a template revision.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (CatalogTemplateRevision) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/templates/{templatesId}/revisions/{revisionsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.catalogs.templates.revisions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsGetRequest',
        response_type_name='CatalogTemplateRevision',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists template revisions.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListCatalogTemplateRevisionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/templates/{templatesId}/revisions',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.catalogs.templates.revisions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/revisions',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsListRequest',
        response_type_name='ListCatalogTemplateRevisionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsSpacesCatalogsTemplatesService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_catalogs_templates resource."""

    _NAME = 'projects_locations_spaces_catalogs_templates'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesCatalogsTemplatesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Adds a template to a catalog.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsTemplatesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (CatalogTemplate) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/templates',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.catalogs.templates.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['catalogTemplateId'],
        relative_path='v1/{+parent}/templates',
        request_field='catalogTemplate',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsTemplatesCreateRequest',
        response_type_name='CatalogTemplate',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a catalog template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsTemplatesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/templates/{templatesId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.catalogs.templates.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsTemplatesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets a catalog template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsTemplatesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (CatalogTemplate) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/templates/{templatesId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.catalogs.templates.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsTemplatesGetRequest',
        response_type_name='CatalogTemplate',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists catalog templates.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsTemplatesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListCatalogTemplatesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/templates',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.catalogs.templates.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/templates',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsTemplatesListRequest',
        response_type_name='ListCatalogTemplatesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a catalog template resource.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsTemplatesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (CatalogTemplate) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}/templates/{templatesId}',
        http_method='PATCH',
        method_id='designcenter.projects.locations.spaces.catalogs.templates.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='catalogTemplate',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsTemplatesPatchRequest',
        response_type_name='CatalogTemplate',
        supports_download=False,
    )

  class ProjectsLocationsSpacesCatalogsService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_catalogs resource."""

    _NAME = 'projects_locations_spaces_catalogs'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesCatalogsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a catalog.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Catalog) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.catalogs.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['catalogId'],
        relative_path='v1/{+parent}/catalogs',
        request_field='catalog',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsCreateRequest',
        response_type_name='Catalog',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a catalog.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.catalogs.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details about a catalog.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Catalog) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.catalogs.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsGetRequest',
        response_type_name='Catalog',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists catalogs in a given space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListCatalogsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.catalogs.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/catalogs',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsListRequest',
        response_type_name='ListCatalogsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a catalog.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCatalogsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Catalog) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/catalogs/{catalogsId}',
        http_method='PATCH',
        method_id='designcenter.projects.locations.spaces.catalogs.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='catalog',
        request_type_name='DesigncenterProjectsLocationsSpacesCatalogsPatchRequest',
        response_type_name='Catalog',
        supports_download=False,
    )

  class ProjectsLocationsSpacesSharedTemplatesRevisionsService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_sharedTemplates_revisions resource."""

    _NAME = 'projects_locations_spaces_sharedTemplates_revisions'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesSharedTemplatesRevisionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets details about a shared template revision.

      Args:
        request: (DesigncenterProjectsLocationsSpacesSharedTemplatesRevisionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SharedTemplateRevision) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/sharedTemplates/{sharedTemplatesId}/revisions/{revisionsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.sharedTemplates.revisions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesSharedTemplatesRevisionsGetRequest',
        response_type_name='SharedTemplateRevision',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists shared template revisions in a given space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesSharedTemplatesRevisionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSharedTemplateRevisionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/sharedTemplates/{sharedTemplatesId}/revisions',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.sharedTemplates.revisions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/revisions',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesSharedTemplatesRevisionsListRequest',
        response_type_name='ListSharedTemplateRevisionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsSpacesSharedTemplatesService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces_sharedTemplates resource."""

    _NAME = 'projects_locations_spaces_sharedTemplates'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesSharedTemplatesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets details about a shared template.

      Args:
        request: (DesigncenterProjectsLocationsSpacesSharedTemplatesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SharedTemplate) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/sharedTemplates/{sharedTemplatesId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.sharedTemplates.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesSharedTemplatesGetRequest',
        response_type_name='SharedTemplate',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists shared templates in a given space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesSharedTemplatesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSharedTemplatesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}/sharedTemplates',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.sharedTemplates.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/sharedTemplates',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesSharedTemplatesListRequest',
        response_type_name='ListSharedTemplatesResponse',
        supports_download=False,
    )

  class ProjectsLocationsSpacesService(base_api.BaseApiService):
    """Service class for the projects_locations_spaces resource."""

    _NAME = 'projects_locations_spaces'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsSpacesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Space) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['spaceId'],
        relative_path='v1/{+parent}/spaces',
        request_field='space',
        request_type_name='DesigncenterProjectsLocationsSpacesCreateRequest',
        response_type_name='Space',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}',
        http_method='DELETE',
        method_id='designcenter.projects.locations.spaces.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def GenerateAssessmentReport(self, request, global_params=None):
      r"""Deprecated: Use GenerateTerraformPlanAssessmentReport instead. Generate a best practice report for a given Terraform plan.

      Args:
        request: (DesigncenterProjectsLocationsSpacesGenerateAssessmentReportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('GenerateAssessmentReport')
      return self._RunMethod(
          config, request, global_params=global_params)

    GenerateAssessmentReport.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}:generateAssessmentReport',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.generateAssessmentReport',
        ordered_params=['spaceName'],
        path_params=['spaceName'],
        query_params=[],
        relative_path='v1/{+spaceName}:generateAssessmentReport',
        request_field='generateAssessmentReportRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesGenerateAssessmentReportRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def GenerateTerraformPlanAssessmentReport(self, request, global_params=None):
      r"""Generate a Terraform plan assessment report for a given Terraform plan.

      Args:
        request: (DesigncenterProjectsLocationsSpacesGenerateTerraformPlanAssessmentReportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('GenerateTerraformPlanAssessmentReport')
      return self._RunMethod(
          config, request, global_params=global_params)

    GenerateTerraformPlanAssessmentReport.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}:generateTerraformPlanAssessmentReport',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.generateTerraformPlanAssessmentReport',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:generateTerraformPlanAssessmentReport',
        request_field='generateTerraformPlanAssessmentReportRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesGenerateTerraformPlanAssessmentReportRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details about a space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Space) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesGetRequest',
        response_type_name='Space',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (DesigncenterProjectsLocationsSpacesGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}:getIamPolicy',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists spaces in a given project and location.

      Args:
        request: (DesigncenterProjectsLocationsSpacesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSpacesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces',
        http_method='GET',
        method_id='designcenter.projects.locations.spaces.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/spaces',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsSpacesListRequest',
        response_type_name='ListSpacesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Space) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}',
        http_method='PATCH',
        method_id='designcenter.projects.locations.spaces.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='space',
        request_type_name='DesigncenterProjectsLocationsSpacesPatchRequest',
        response_type_name='Space',
        supports_download=False,
    )

    def RegisterAppHubApplicationResources(self, request, global_params=None):
      r"""Register deployed resources from terraform state file with App Hub without needing a space.

      Args:
        request: (DesigncenterProjectsLocationsSpacesRegisterAppHubApplicationResourcesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RegisterAppHubApplicationResources')
      return self._RunMethod(
          config, request, global_params=global_params)

    RegisterAppHubApplicationResources.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces:registerAppHubApplicationResources',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.registerAppHubApplicationResources',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1/{+parent}/spaces:registerAppHubApplicationResources',
        request_field='registerAppHubApplicationResourcesRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesRegisterAppHubApplicationResourcesRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy. Can return `NOT_FOUND`, `INVALID_ARGUMENT`, and `PERMISSION_DENIED` errors.

      Args:
        request: (DesigncenterProjectsLocationsSpacesSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}:setIamPolicy',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a `NOT_FOUND` error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (DesigncenterProjectsLocationsSpacesTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/spaces/{spacesId}:testIamPermissions',
        http_method='POST',
        method_id='designcenter.projects.locations.spaces.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='DesigncenterProjectsLocationsSpacesTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def FetchFrameworks(self, request, global_params=None):
      r"""Fetches all frameworks for assessment report.

      Args:
        request: (DesigncenterProjectsLocationsFetchFrameworksRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FetchFrameworksResponse) The response message.
      """
      config = self.GetMethodConfig('FetchFrameworks')
      return self._RunMethod(
          config, request, global_params=global_params)

    FetchFrameworks.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}:fetchFrameworks',
        http_method='GET',
        method_id='designcenter.projects.locations.fetchFrameworks',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken', 'projects'],
        relative_path='v1/{+parent}:fetchFrameworks',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsFetchFrameworksRequest',
        response_type_name='FetchFrameworksResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (DesigncenterProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='designcenter.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (DesigncenterProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations',
        http_method='GET',
        method_id='designcenter.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+name}/locations',
        request_field='',
        request_type_name='DesigncenterProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(DesigncenterV1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



#!/usr/bin/env python
#
# This file is part of pyasn1 software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://pyasn1.readthedocs.io/en/latest/license.html
#
import warnings

from pyasn1.codec.cer import decoder
from pyasn1.type import univ

__all__ = ['decode', 'StreamingDecoder']


class BitStringPayloadDecoder(decoder.BitStringPayloadDecoder):
    supportConstructedForm = False


class OctetStringPayloadDecoder(decoder.OctetStringPayloadDecoder):
    supportConstructedForm = False


# TODO: prohibit non-canonical encoding
RealPayloadDecoder = decoder.RealPayloadDecoder

TAG_MAP = decoder.TAG_MAP.copy()
TAG_MAP.update(
    {univ.BitString.tagSet: BitStringPayloadDecoder(),
     univ.OctetString.tagSet: OctetStringPayloadDecoder(),
     univ.Real.tagSet: RealPayloadDecoder()}
)

TYPE_MAP = decoder.TYPE_MAP.copy()

# Put in non-ambiguous types for faster codec lookup
for typeDecoder in TAG_MAP.values():
    if typeDecoder.protoComponent is not None:
        typeId = typeDecoder.protoComponent.__class__.typeId
        if typeId is not None and typeId not in TYPE_MAP:
            TYPE_MAP[typeId] = typeDecoder


class SingleItemDecoder(decoder.SingleItemDecoder):
    __doc__ = decoder.SingleItemDecoder.__doc__

    TAG_MAP = TAG_MAP
    TYPE_MAP = TYPE_MAP

    supportIndefLength = False


class StreamingDecoder(decoder.StreamingDecoder):
    __doc__ = decoder.StreamingDecoder.__doc__

    SINGLE_ITEM_DECODER = SingleItemDecoder


class Decoder(decoder.Decoder):
    __doc__ = decoder.Decoder.__doc__

    STREAMING_DECODER = StreamingDecoder


#: Turns DER octet stream into an ASN.1 object.
#:
#: Takes DER octet-stream and decode it into an ASN.1 object
#: (e.g. :py:class:`~pyasn1.type.base.PyAsn1Item` derivative) which
#: may be a scalar or an arbitrary nested structure.
#:
#: Parameters
#: ----------
#: substrate: :py:class:`bytes`
#:     DER octet-stream
#:
#: Keyword Args
#: ------------
#: asn1Spec: any pyasn1 type object e.g. :py:class:`~pyasn1.type.base.PyAsn1Item` derivative
#:     A pyasn1 type object to act as a template guiding the decoder. Depending on the ASN.1 structure
#:     being decoded, *asn1Spec* may or may not be required. Most common reason for
#:     it to require is that ASN.1 structure is encoded in *IMPLICIT* tagging mode.
#:
#: Returns
#: -------
#: : :py:class:`tuple`
#:     A tuple of pyasn1 object recovered from DER substrate (:py:class:`~pyasn1.type.base.PyAsn1Item` derivative)
#:     and the unprocessed trailing portion of the *substrate* (may be empty)
#:
#: Raises
#: ------
#: ~pyasn1.error.PyAsn1Error, ~pyasn1.error.SubstrateUnderrunError
#:     On decoding errors
#:
#: Examples
#: --------
#: Decode DER serialisation without ASN.1 schema
#:
#: .. code-block:: pycon
#:
#:    >>> s, _ = decode(b'0\t\x02\x01\x01\x02\x01\x02\x02\x01\x03')
#:    >>> str(s)
#:    SequenceOf:
#:     1 2 3
#:
#: Decode DER serialisation with ASN.1 schema
#:
#: .. code-block:: pycon
#:
#:    >>> seq = SequenceOf(componentType=Integer())
#:    >>> s, _ = decode(b'0\t\x02\x01\x01\x02\x01\x02\x02\x01\x03', asn1Spec=seq)
#:    >>> str(s)
#:    SequenceOf:
#:     1 2 3
#:
decode = Decoder()

def __getattr__(attr: str):
    if newAttr := {"tagMap": "TAG_MAP", "typeMap": "TYPE_MAP"}.get(attr):
        warnings.warn(f"{attr} is deprecated. Please use {newAttr} instead.", DeprecationWarning, stacklevel=2)
        return globals()[newAttr]
    raise AttributeError(attr)
